@@ -1,24 +1,22 @@
-# -*- coding: utf-8 -*-
 """
     :codeauthor: :email:`Bo Maryniuk <bo@suse.de>`
 """
-from __future__ import absolute_import, print_function, unicode_literals
 
 import copy
 import os
+import pathlib
 import shutil
 import sys
 import tarfile
 import tempfile
 
 import jinja2
+import pytest
 import salt.exceptions
-import salt.ext.six
 import salt.utils.hashutils
 import salt.utils.json
 import salt.utils.platform
 import salt.utils.stringutils
-from salt.ext.six.moves import range
 from salt.utils import thin
 from salt.utils.stringutils import to_bytes as bts
 from tests.support.helpers import TstSuiteLoggingHandler, VirtualEnv
@@ -32,6 +30,19 @@ except ImportError:
     pytest = None
 
 
+def patch_if(condition, *args, **kwargs):
+    """
+    Return a patch decorator if the provided condition is met
+    """
+    if condition:
+        return patch(*args, **kwargs)
+
+    def inner(func):
+        return func
+
+    return inner
+
+
 @skipIf(pytest is None, "PyTest is missing")
 class SSHThinTestCase(TestCase):
     """
@@ -44,7 +55,7 @@ class SSHThinTestCase(TestCase):
         self.ext_conf = {
             "test": {
                 "py-version": [2, 7],
-                "path": os.path.join(RUNTIME_VARS.CODE_DIR, "salt"),
+                "path": RUNTIME_VARS.SALT_CODE_DIR,
                 "dependencies": {"jinja2": self.jinja_fp},
             }
         }
@@ -65,18 +76,15 @@ class SSHThinTestCase(TestCase):
             "msgpack": os.path.join(lib_root, "msgpack"),
         }
 
+        code_dir = pathlib.Path(RUNTIME_VARS.CODE_DIR).resolve()
         self.exp_ret = {
-            "distro": os.path.normpath(
-                os.path.join(RUNTIME_VARS.CODE_DIR, "distro.py")
-            ),
-            "jinja2": os.path.normpath(os.path.join(RUNTIME_VARS.CODE_DIR, "jinja2")),
-            "yaml": os.path.normpath(os.path.join(RUNTIME_VARS.CODE_DIR, "yaml")),
-            "tornado": os.path.normpath(os.path.join(RUNTIME_VARS.CODE_DIR, "tornado")),
-            "msgpack": os.path.normpath(os.path.join(RUNTIME_VARS.CODE_DIR, "msgpack")),
-            "certifi": os.path.normpath(os.path.join(RUNTIME_VARS.CODE_DIR, "certifi")),
-            "singledispatch": os.path.normpath(
-                os.path.join(RUNTIME_VARS.CODE_DIR, "singledispatch.py")
-            ),
+            "distro": str(code_dir / "distro.py"),
+            "jinja2": str(code_dir / "jinja2"),
+            "yaml": str(code_dir / "yaml"),
+            "tornado": str(code_dir / "tornado"),
+            "msgpack": str(code_dir / "msgpack"),
+            "certifi": str(code_dir / "certifi"),
+            "singledispatch": str(code_dir / "singledispatch.py"),
         }
         self.exc_libs = ["jinja2", "yaml"]
 
@@ -141,7 +149,6 @@ class SSHThinTestCase(TestCase):
 
         return tf
 
-    @patch("salt.exceptions.SaltSystemExit", Exception)
     @patch("salt.utils.thin.log", MagicMock())
     @patch("salt.utils.thin.os.path.isfile", MagicMock(return_value=False))
     def test_get_ext_tops_cfg_missing_dependencies(self):
@@ -152,7 +159,7 @@ class SSHThinTestCase(TestCase):
         """
         cfg = {"namespace": {"py-version": [0, 0], "path": "/foo", "dependencies": []}}
 
-        with pytest.raises(Exception) as err:
+        with pytest.raises(salt.exceptions.SaltSystemExit) as err:
             thin.get_ext_tops(cfg)
         self.assertIn("Missing dependencies", str(err.value))
         self.assertTrue(thin.log.error.called)
@@ -227,7 +234,6 @@ class SSHThinTestCase(TestCase):
             == thin.log.warning.mock_calls[0][1][0] % "test"
         )
 
-    @patch("salt.exceptions.SaltSystemExit", Exception)
     @patch("salt.utils.thin.log", MagicMock())
     @patch("salt.utils.thin.os.path.isfile", MagicMock(return_value=False))
     def test_get_ext_tops_dependency_config_check(self):
@@ -319,7 +325,7 @@ class SSHThinTestCase(TestCase):
         """
         container = []
         for pth in ["/foo/bar.py", "/something/else/__init__.py"]:
-            thin._add_dependency(container, type(str("obj"), (), {"__file__": pth})())
+            thin._add_dependency(container, type("obj", (), {"__file__": pth})())
         assert "__init__" not in container[1]
         assert container == ["/foo/bar.py", "/something/else"]
 
@@ -381,52 +387,59 @@ class SSHThinTestCase(TestCase):
         type("distro", (), {"__file__": "/site-packages/distro"}),
     )
     @patch(
-        "salt.utils.thin.salt",
-        type(str("salt"), (), {"__file__": "/site-packages/salt"}),
+        "salt.utils.thin.salt", type("salt", (), {"__file__": "/site-packages/salt"}),
     )
     @patch(
         "salt.utils.thin.jinja2",
-        type(str("jinja2"), (), {"__file__": "/site-packages/jinja2"}),
+        type("jinja2", (), {"__file__": "/site-packages/jinja2"}),
     )
     @patch(
-        "salt.utils.thin.yaml",
-        type(str("yaml"), (), {"__file__": "/site-packages/yaml"}),
+        "salt.utils.thin.yaml", type("yaml", (), {"__file__": "/site-packages/yaml"}),
     )
     @patch(
         "salt.utils.thin.tornado",
-        type(str("tornado"), (), {"__file__": "/site-packages/tornado"}),
+        type("tornado", (), {"__file__": "/site-packages/tornado"}),
     )
     @patch(
         "salt.utils.thin.msgpack",
-        type(str("msgpack"), (), {"__file__": "/site-packages/msgpack"}),
+        type("msgpack", (), {"__file__": "/site-packages/msgpack"}),
     )
     @patch(
         "salt.utils.thin.certifi",
-        type(str("certifi"), (), {"__file__": "/site-packages/certifi"}),
+        type("certifi", (), {"__file__": "/site-packages/certifi"}),
     )
     @patch(
         "salt.utils.thin.singledispatch",
-        type(str("singledispatch"), (), {"__file__": "/site-packages/sdp"}),
+        type("singledispatch", (), {"__file__": "/site-packages/sdp"}),
     )
     @patch(
         "salt.utils.thin.singledispatch_helpers",
-        type(str("singledispatch_helpers"), (), {"__file__": "/site-packages/sdp_hlp"}),
+        type("singledispatch_helpers", (), {"__file__": "/site-packages/sdp_hlp"}),
     )
     @patch(
         "salt.utils.thin.ssl_match_hostname",
-        type(str("ssl_match_hostname"), (), {"__file__": "/site-packages/ssl_mh"}),
+        type("ssl_match_hostname", (), {"__file__": "/site-packages/ssl_mh"}),
     )
     @patch(
         "salt.utils.thin.markupsafe",
-        type(str("markupsafe"), (), {"__file__": "/site-packages/markupsafe"}),
+        type("markupsafe", (), {"__file__": "/site-packages/markupsafe"}),
     )
     @patch(
         "salt.utils.thin.backports_abc",
-        type(str("backports_abc"), (), {"__file__": "/site-packages/backports_abc"}),
+        type("backports_abc", (), {"__file__": "/site-packages/backports_abc"}),
     )
     @patch(
         "salt.utils.thin.concurrent",
-        type(str("concurrent"), (), {"__file__": "/site-packages/concurrent"}),
+        type("concurrent", (), {"__file__": "/site-packages/concurrent"}),
+    )
+    @patch(
+        "salt.utils.thin.py_contextvars",
+        type("contextvars", (), {"__file__": "/site-packages/contextvars"}),
+    )
+    @patch_if(
+        salt.utils.thin.has_immutables,
+        "salt.utils.thin.immutables",
+        type("immutables", (), {"__file__": "/site-packages/immutables"}),
     )
     @patch("salt.utils.thin.log", MagicMock())
     def test_get_tops(self):
@@ -435,76 +448,91 @@ class SSHThinTestCase(TestCase):
         :return:
         """
         base_tops = [
-            "/site-packages/distro",
-            "/site-packages/salt",
-            "/site-packages/jinja2",
-            "/site-packages/yaml",
-            "/site-packages/tornado",
-            "/site-packages/msgpack",
-            "/site-packages/certifi",
-            "/site-packages/sdp",
-            "/site-packages/sdp_hlp",
-            "/site-packages/ssl_mh",
-            "/site-packages/markupsafe",
-            "/site-packages/backports_abc",
-            "/site-packages/concurrent",
+            "distro",
+            "salt",
+            "jinja2",
+            "yaml",
+            "tornado",
+            "msgpack",
+            "certifi",
+            "sdp",
+            "sdp_hlp",
+            "ssl_mh",
+            "markupsafe",
+            "backports_abc",
+            "concurrent",
+            "contextvars",
         ]
-
-        tops = thin.get_tops()
+        if salt.utils.thin.has_immutables:
+            base_tops.extend(["immutables"])
+        tops = []
+        for top in thin.get_tops(extra_mods="foo,bar"):
+            if top.find("/") != -1:
+                spl = "/"
+            else:
+                spl = os.sep
+            tops.append(top.rsplit(spl, 1)[-1])
         assert len(tops) == len(base_tops)
-        assert sorted(tops) == sorted(base_tops)
+        assert sorted(tops) == sorted(base_tops), sorted(tops)
 
     @patch(
         "salt.utils.thin.distro",
         type("distro", (), {"__file__": "/site-packages/distro"}),
     )
     @patch(
-        "salt.utils.thin.salt",
-        type(str("salt"), (), {"__file__": "/site-packages/salt"}),
+        "salt.utils.thin.salt", type("salt", (), {"__file__": "/site-packages/salt"}),
     )
     @patch(
         "salt.utils.thin.jinja2",
-        type(str("jinja2"), (), {"__file__": "/site-packages/jinja2"}),
+        type("jinja2", (), {"__file__": "/site-packages/jinja2"}),
     )
     @patch(
-        "salt.utils.thin.yaml",
-        type(str("yaml"), (), {"__file__": "/site-packages/yaml"}),
+        "salt.utils.thin.yaml", type("yaml", (), {"__file__": "/site-packages/yaml"}),
     )
     @patch(
         "salt.utils.thin.tornado",
-        type(str("tornado"), (), {"__file__": "/site-packages/tornado"}),
+        type("tornado", (), {"__file__": "/site-packages/tornado"}),
     )
     @patch(
         "salt.utils.thin.msgpack",
-        type(str("msgpack"), (), {"__file__": "/site-packages/msgpack"}),
+        type("msgpack", (), {"__file__": "/site-packages/msgpack"}),
     )
     @patch(
         "salt.utils.thin.certifi",
-        type(str("certifi"), (), {"__file__": "/site-packages/certifi"}),
+        type("certifi", (), {"__file__": "/site-packages/certifi"}),
     )
     @patch(
         "salt.utils.thin.singledispatch",
-        type(str("singledispatch"), (), {"__file__": "/site-packages/sdp"}),
+        type("singledispatch", (), {"__file__": "/site-packages/sdp"}),
     )
     @patch(
         "salt.utils.thin.singledispatch_helpers",
-        type(str("singledispatch_helpers"), (), {"__file__": "/site-packages/sdp_hlp"}),
+        type("singledispatch_helpers", (), {"__file__": "/site-packages/sdp_hlp"}),
     )
     @patch(
         "salt.utils.thin.ssl_match_hostname",
-        type(str("ssl_match_hostname"), (), {"__file__": "/site-packages/ssl_mh"}),
+        type("ssl_match_hostname", (), {"__file__": "/site-packages/ssl_mh"}),
     )
     @patch(
         "salt.utils.thin.markupsafe",
-        type(str("markupsafe"), (), {"__file__": "/site-packages/markupsafe"}),
+        type("markupsafe", (), {"__file__": "/site-packages/markupsafe"}),
     )
     @patch(
         "salt.utils.thin.backports_abc",
-        type(str("backports_abc"), (), {"__file__": "/site-packages/backports_abc"}),
+        type("backports_abc", (), {"__file__": "/site-packages/backports_abc"}),
     )
     @patch(
         "salt.utils.thin.concurrent",
-        type(str("concurrent"), (), {"__file__": "/site-packages/concurrent"}),
+        type("concurrent", (), {"__file__": "/site-packages/concurrent"}),
+    )
+    @patch(
+        "salt.utils.thin.py_contextvars",
+        type("contextvars", (), {"__file__": "/site-packages/contextvars"}),
+    )
+    @patch_if(
+        salt.utils.thin.has_immutables,
+        "salt.utils.thin.immutables",
+        type("immutables", (), {"__file__": "/site-packages/immutables"}),
     )
     @patch("salt.utils.thin.log", MagicMock())
     def test_get_tops_extra_mods(self):
@@ -513,32 +541,40 @@ class SSHThinTestCase(TestCase):
         :return:
         """
         base_tops = [
-            "/site-packages/distro",
-            "/site-packages/salt",
-            "/site-packages/jinja2",
-            "/site-packages/yaml",
-            "/site-packages/tornado",
-            "/site-packages/msgpack",
-            "/site-packages/certifi",
-            "/site-packages/sdp",
-            "/site-packages/sdp_hlp",
-            "/site-packages/ssl_mh",
-            "/site-packages/concurrent",
-            "/site-packages/markupsafe",
-            "/site-packages/backports_abc",
-            os.sep + os.path.join("custom", "foo"),
-            os.sep + os.path.join("custom", "bar.py"),
+            "distro",
+            "salt",
+            "jinja2",
+            "yaml",
+            "tornado",
+            "msgpack",
+            "certifi",
+            "sdp",
+            "sdp_hlp",
+            "ssl_mh",
+            "concurrent",
+            "markupsafe",
+            "backports_abc",
+            "contextvars",
+            "foo",
+            "bar.py",
         ]
-        builtins = sys.version_info.major == 3 and "builtins" or "__builtin__"
+        if salt.utils.thin.has_immutables:
+            base_tops.extend(["immutables"])
+        libs = salt.utils.thin.find_site_modules("contextvars")
         foo = {"__file__": os.sep + os.path.join("custom", "foo", "__init__.py")}
         bar = {"__file__": os.sep + os.path.join("custom", "bar")}
-        with patch(
-            "{}.__import__".format(builtins),
-            MagicMock(
-                side_effect=[type(str("foo"), (), foo), type(str("bar"), (), bar)]
-            ),
-        ):
-            tops = thin.get_tops(extra_mods="foo,bar")
+        with patch("salt.utils.thin.find_site_modules", MagicMock(side_effect=[libs])):
+            with patch(
+                "builtins.__import__",
+                MagicMock(side_effect=[type("foo", (), foo), type("bar", (), bar)]),
+            ):
+                tops = []
+                for top in thin.get_tops(extra_mods="foo,bar"):
+                    if top.find("/") != -1:
+                        spl = "/"
+                    else:
+                        spl = os.sep
+                    tops.append(top.rsplit(spl, 1)[-1])
         self.assertEqual(len(tops), len(base_tops))
         self.assertListEqual(sorted(tops), sorted(base_tops))
 
@@ -547,52 +583,59 @@ class SSHThinTestCase(TestCase):
         type("distro", (), {"__file__": "/site-packages/distro"}),
     )
     @patch(
-        "salt.utils.thin.salt",
-        type(str("salt"), (), {"__file__": "/site-packages/salt"}),
+        "salt.utils.thin.salt", type("salt", (), {"__file__": "/site-packages/salt"}),
     )
     @patch(
         "salt.utils.thin.jinja2",
-        type(str("jinja2"), (), {"__file__": "/site-packages/jinja2"}),
+        type("jinja2", (), {"__file__": "/site-packages/jinja2"}),
     )
     @patch(
-        "salt.utils.thin.yaml",
-        type(str("yaml"), (), {"__file__": "/site-packages/yaml"}),
+        "salt.utils.thin.yaml", type("yaml", (), {"__file__": "/site-packages/yaml"}),
     )
     @patch(
         "salt.utils.thin.tornado",
-        type(str("tornado"), (), {"__file__": "/site-packages/tornado"}),
+        type("tornado", (), {"__file__": "/site-packages/tornado"}),
     )
     @patch(
         "salt.utils.thin.msgpack",
-        type(str("msgpack"), (), {"__file__": "/site-packages/msgpack"}),
+        type("msgpack", (), {"__file__": "/site-packages/msgpack"}),
     )
     @patch(
         "salt.utils.thin.certifi",
-        type(str("certifi"), (), {"__file__": "/site-packages/certifi"}),
+        type("certifi", (), {"__file__": "/site-packages/certifi"}),
     )
     @patch(
         "salt.utils.thin.singledispatch",
-        type(str("singledispatch"), (), {"__file__": "/site-packages/sdp"}),
+        type("singledispatch", (), {"__file__": "/site-packages/sdp"}),
     )
     @patch(
         "salt.utils.thin.singledispatch_helpers",
-        type(str("singledispatch_helpers"), (), {"__file__": "/site-packages/sdp_hlp"}),
+        type("singledispatch_helpers", (), {"__file__": "/site-packages/sdp_hlp"}),
     )
     @patch(
         "salt.utils.thin.ssl_match_hostname",
-        type(str("ssl_match_hostname"), (), {"__file__": "/site-packages/ssl_mh"}),
+        type("ssl_match_hostname", (), {"__file__": "/site-packages/ssl_mh"}),
     )
     @patch(
         "salt.utils.thin.markupsafe",
-        type(str("markupsafe"), (), {"__file__": "/site-packages/markupsafe"}),
+        type("markupsafe", (), {"__file__": "/site-packages/markupsafe"}),
     )
     @patch(
         "salt.utils.thin.backports_abc",
-        type(str("backports_abc"), (), {"__file__": "/site-packages/backports_abc"}),
+        type("backports_abc", (), {"__file__": "/site-packages/backports_abc"}),
     )
     @patch(
         "salt.utils.thin.concurrent",
-        type(str("concurrent"), (), {"__file__": "/site-packages/concurrent"}),
+        type("concurrent", (), {"__file__": "/site-packages/concurrent"}),
+    )
+    @patch(
+        "salt.utils.thin.py_contextvars",
+        type("contextvars", (), {"__file__": "/site-packages/contextvars"}),
+    )
+    @patch_if(
+        salt.utils.thin.has_immutables,
+        "salt.utils.thin.immutables",
+        type("immutables", (), {"__file__": "/site-packages/immutables"}),
     )
     @patch("salt.utils.thin.log", MagicMock())
     def test_get_tops_so_mods(self):
@@ -601,33 +644,43 @@ class SSHThinTestCase(TestCase):
         :return:
         """
         base_tops = [
-            "/site-packages/distro",
-            "/site-packages/salt",
-            "/site-packages/jinja2",
-            "/site-packages/yaml",
-            "/site-packages/tornado",
-            "/site-packages/msgpack",
-            "/site-packages/certifi",
-            "/site-packages/sdp",
-            "/site-packages/sdp_hlp",
-            "/site-packages/ssl_mh",
-            "/site-packages/concurrent",
-            "/site-packages/markupsafe",
-            "/site-packages/backports_abc",
-            "/custom/foo.so",
-            "/custom/bar.so",
+            "distro",
+            "salt",
+            "jinja2",
+            "yaml",
+            "tornado",
+            "msgpack",
+            "certifi",
+            "sdp",
+            "sdp_hlp",
+            "ssl_mh",
+            "concurrent",
+            "markupsafe",
+            "backports_abc",
+            "contextvars",
+            "foo.so",
+            "bar.so",
         ]
-        builtins = sys.version_info.major == 3 and "builtins" or "__builtin__"
-        with patch(
-            "{}.__import__".format(builtins),
-            MagicMock(
-                side_effect=[
-                    type(str("salt"), (), {"__file__": "/custom/foo.so"}),
-                    type(str("salt"), (), {"__file__": "/custom/bar.so"}),
-                ]
-            ),
-        ):
-            tops = thin.get_tops(so_mods="foo,bar")
+        if salt.utils.thin.has_immutables:
+            base_tops.extend(["immutables"])
+        libs = salt.utils.thin.find_site_modules("contextvars")
+        with patch("salt.utils.thin.find_site_modules", MagicMock(side_effect=[libs])):
+            with patch(
+                "builtins.__import__",
+                MagicMock(
+                    side_effect=[
+                        type("salt", (), {"__file__": "/custom/foo.so"}),
+                        type("salt", (), {"__file__": "/custom/bar.so"}),
+                    ]
+                ),
+            ):
+                tops = []
+                for top in thin.get_tops(so_mods="foo,bar"):
+                    if top.find("/") != -1:
+                        spl = "/"
+                    else:
+                        spl = os.sep
+                    tops.append(top.rsplit(spl, 1)[-1])
         assert len(tops) == len(base_tops)
         assert sorted(tops) == sorted(base_tops)
 
@@ -676,75 +729,10 @@ class SSHThinTestCase(TestCase):
             thin.sys.exc_clear = lambda: None
             thin.gen_thin("")
         self.assertIn(
-            "The minimum required python version to run salt-ssh is " '"2.6"',
+            "The minimum required python version to run salt-ssh is " '"3"',
             str(err.value),
         )
 
-    @skipIf(
-        salt.utils.platform.is_windows() and thin._six.PY2, "Dies on Python2 on Windows"
-    )
-    @patch("salt.exceptions.SaltSystemExit", Exception)
-    @patch("salt.utils.thin.os.makedirs", MagicMock())
-    @patch("salt.utils.files.fopen", MagicMock())
-    @patch("salt.utils.thin._get_salt_call", MagicMock())
-    @patch("salt.utils.thin._get_ext_namespaces", MagicMock())
-    @patch("salt.utils.thin.get_tops", MagicMock(return_value=["/foo3", "/bar3"]))
-    @patch("salt.utils.thin.get_ext_tops", MagicMock(return_value={}))
-    @patch("salt.utils.thin.os.path.isfile", MagicMock())
-    @patch("salt.utils.thin.os.path.isdir", MagicMock(return_value=True))
-    @patch("salt.utils.thin.os.remove", MagicMock())
-    @patch("salt.utils.thin.os.path.exists", MagicMock())
-    @patch("salt.utils.path.os_walk", MagicMock(return_value=[]))
-    @patch(
-        "salt.utils.thin.subprocess.Popen",
-        _popen(
-            None,
-            side_effect=[(bts("2.7"), bts("")), (bts('["/foo27", "/bar27"]'), bts(""))],
-        ),
-    )
-    @patch("salt.utils.thin.tarfile", MagicMock())
-    @patch("salt.utils.thin.zipfile", MagicMock())
-    @patch("salt.utils.thin.os.getcwd", MagicMock())
-    @patch("salt.utils.thin.os.access", MagicMock(return_value=True))
-    @patch("salt.utils.thin.os.chdir", MagicMock())
-    @patch("salt.utils.thin.tempfile.mkdtemp", MagicMock())
-    @patch(
-        "salt.utils.thin.tempfile.mkstemp", MagicMock(return_value=(3, ".temporary"))
-    )
-    @patch("salt.utils.thin.shutil", MagicMock())
-    @patch("salt.utils.path.which", MagicMock(return_value=""))
-    @patch("salt.utils.thin._get_thintar_prefix", MagicMock())
-    def test_gen_thin_python_exist_or_not(self):
-        """
-        Test thin.gen_thin function if the opposite python
-        binary does not exist
-        """
-        with TstSuiteLoggingHandler() as handler:
-            thin.gen_thin("")
-            salt.utils.thin.subprocess.Popen.assert_not_called()
-
-            if salt.ext.six.PY2:
-                self.assertIn(
-                    "DEBUG:python3 binary does not exist. Will not attempt to generate "
-                    "tops for Python 3",
-                    handler.messages,
-                )
-
-            if salt.ext.six.PY3:
-                self.assertIn(
-                    "DEBUG:python2 binary does not exist. Will not "
-                    "detect Python 2 version",
-                    handler.messages,
-                )
-                self.assertIn(
-                    "DEBUG:python2 binary does not exist. Will not attempt to generate "
-                    "tops for Python 2",
-                    handler.messages,
-                )
-
-    @skipIf(
-        salt.utils.platform.is_windows() and thin._six.PY2, "Dies on Python2 on Windows"
-    )
     @patch("salt.exceptions.SaltSystemExit", Exception)
     @patch("salt.utils.thin.log", MagicMock())
     @patch("salt.utils.thin.os.makedirs", MagicMock())
@@ -777,8 +765,6 @@ class SSHThinTestCase(TestCase):
         "salt.utils.thin.tempfile.mkstemp", MagicMock(return_value=(3, ".temporary"))
     )
     @patch("salt.utils.thin.shutil", MagicMock())
-    @patch("salt.utils.thin._six.PY3", True)
-    @patch("salt.utils.thin._six.PY2", False)
     @patch("salt.utils.thin.sys.version_info", _version_info(None, 3, 6))
     @patch("salt.utils.path.which", MagicMock(return_value="/usr/bin/python"))
     def test_gen_thin_compression_fallback_py3(self):
@@ -830,15 +816,11 @@ class SSHThinTestCase(TestCase):
         "salt.utils.thin.tempfile.mkstemp", MagicMock(return_value=(3, ".temporary"))
     )
     @patch("salt.utils.thin.shutil", MagicMock())
-    @patch("salt.utils.thin._six.PY3", True)
-    @patch("salt.utils.thin._six.PY2", False)
     @patch("salt.utils.thin.sys.version_info", _version_info(None, 3, 6))
     @patch("salt.utils.path.which", MagicMock(return_value="/usr/bin/python"))
     def test_gen_thin_control_files_written_py3(self):
         """
         Test thin.gen_thin function if control files are written (version, salt-call etc).
-        NOTE: Py2 version of this test is not required, as code shares the same spot across the versions.
-
         :return:
         """
         thin.gen_thin("")
@@ -848,7 +830,7 @@ class SSHThinTestCase(TestCase):
         for idx, fname in enumerate(
             ["version", ".thin-gen-py-version", "salt-call", "supported-versions"]
         ):
-            name = thin.tarfile.open().method_calls[idx + 4][1][0]
+            name = thin.tarfile.open().method_calls[idx + 2][1][0]
             self.assertEqual(name, fname)
         thin.tarfile.open().close.assert_called()
 
@@ -874,13 +856,6 @@ class SSHThinTestCase(TestCase):
             )
         ),
     )
-    @patch(
-        "salt.utils.thin.subprocess.Popen",
-        _popen(
-            None,
-            side_effect=[(bts("2.7"), bts("")), (bts('["/foo27", "/bar27"]'), bts(""))],
-        ),
-    )
     @patch("salt.utils.thin.tarfile", _tarfile(None))
     @patch("salt.utils.thin.zipfile", MagicMock())
     @patch(
@@ -894,8 +869,6 @@ class SSHThinTestCase(TestCase):
         "salt.utils.thin.tempfile.mkstemp", MagicMock(return_value=(3, ".temporary"))
     )
     @patch("salt.utils.thin.shutil", MagicMock())
-    @patch("salt.utils.thin._six.PY3", True)
-    @patch("salt.utils.thin._six.PY2", False)
     @patch("salt.utils.thin.sys.version_info", _version_info(None, 3, 6))
     @patch("salt.utils.hashutils.DigestCollector", MagicMock())
     @patch("salt.utils.path.which", MagicMock(return_value="/usr/bin/python"))
@@ -908,11 +881,11 @@ class SSHThinTestCase(TestCase):
         """
         thin.gen_thin("")
         files = []
-        for py in ("py2", "py2", "py3", "pyall"):
+        for py in ("py3", "pyall"):
             for i in range(1, 4):
-                files.append(os.path.join(py, "root", "r{0}".format(i)))
+                files.append(os.path.join(py, "root", "r{}".format(i)))
             for i in range(4, 7):
-                files.append(os.path.join(py, "root2", "r{0}".format(i)))
+                files.append(os.path.join(py, "root2", "r{}".format(i)))
         for cl in thin.tarfile.open().method_calls[:-6]:
             arcname = cl[2].get("arcname")
             self.assertIn(arcname, files)
@@ -931,7 +904,7 @@ class SSHThinTestCase(TestCase):
         MagicMock(
             return_value={
                 "namespace": {
-                    "py-version": [2, 7],
+                    "py-version": [3, 0],
                     "path": "/opt/2015.8/salt",
                     "dependencies": ["/opt/certifi", "/opt/whatever"],
                 }
@@ -952,13 +925,6 @@ class SSHThinTestCase(TestCase):
             )
         ),
     )
-    @patch(
-        "salt.utils.thin.subprocess.Popen",
-        _popen(
-            None,
-            side_effect=[(bts("2.7"), bts("")), (bts('["/foo27", "/bar27"]'), bts(""))],
-        ),
-    )
     @patch("salt.utils.thin.tarfile", _tarfile(None))
     @patch("salt.utils.thin.zipfile", MagicMock())
     @patch(
@@ -972,21 +938,17 @@ class SSHThinTestCase(TestCase):
         "salt.utils.thin.tempfile.mkstemp", MagicMock(return_value=(3, ".temporary"))
     )
     @patch("salt.utils.thin.shutil", MagicMock())
-    @patch("salt.utils.thin._six.PY3", True)
-    @patch("salt.utils.thin._six.PY2", False)
     @patch("salt.utils.thin.sys.version_info", _version_info(None, 3, 6))
     @patch("salt.utils.hashutils.DigestCollector", MagicMock())
     @patch("salt.utils.path.which", MagicMock(return_value="/usr/bin/python"))
     def test_gen_thin_ext_alternative_content_files_written_py3(self):
         """
         Test thin.gen_thin function if external alternative content files are written.
-        NOTE: Py2 version of this test is not required, as code shares the same spot across the versions.
-
         :return:
         """
         ext_conf = {
             "namespace": {
-                "py-version": [2, 7],
+                "py-version": [3, 0],
                 "path": "/opt/2015.8/salt",
                 "dependencies": {
                     "certifi": "/opt/certifi",
@@ -997,13 +959,13 @@ class SSHThinTestCase(TestCase):
 
         thin.gen_thin("", extended_cfg=ext_conf)
         files = []
-        for py in ("pyall", "pyall", "py2"):
+        for py in ("pyall", "pyall", "py3"):
             for i in range(1, 4):
-                files.append(os.path.join("namespace", py, "root", "r{0}".format(i)))
+                files.append(os.path.join("namespace", py, "root", "r{}".format(i)))
             for i in range(4, 7):
-                files.append(os.path.join("namespace", py, "root2", "r{0}".format(i)))
+                files.append(os.path.join("namespace", py, "root2", "r{}".format(i)))
 
-        for idx, cl in enumerate(thin.tarfile.open().method_calls[12:-6]):
+        for idx, cl in enumerate(thin.tarfile.open().method_calls[:-6]):
             arcname = cl[2].get("arcname")
             self.assertIn(arcname, files)
             files.pop(files.index(arcname))
@@ -1089,8 +1051,6 @@ class SSHThinTestCase(TestCase):
         "salt.utils.thin.tempfile.mkstemp", MagicMock(return_value=(3, ".temporary"))
     )
     @patch("salt.utils.thin.shutil", MagicMock())
-    @patch("salt.utils.thin._six.PY3", True)
-    @patch("salt.utils.thin._six.PY2", False)
     @patch("salt.utils.thin.sys.version_info", _version_info(None, 3, 6))
     def test_gen_thin_control_files_written_access_denied_cwd(self):
         """
@@ -1123,7 +1083,6 @@ class SSHThinTestCase(TestCase):
             self._popen(
                 None,
                 side_effect=[
-                    (bts("distro.py"), bts("")),
                     (bts("jinja2/__init__.py"), bts("")),
                     (bts("yaml/__init__.py"), bts("")),
                     (bts("tornado/__init__.py"), bts("")),
@@ -1135,6 +1094,7 @@ class SSHThinTestCase(TestCase):
                     (bts(""), bts("")),
                     (bts(""), bts("")),
                     (bts(""), bts("")),
+                    (bts("distro.py"), bts("")),
                 ],
             ),
         )
@@ -1143,10 +1103,16 @@ class SSHThinTestCase(TestCase):
         patch_which = patch("salt.utils.path.which", return_value=True)
         with patch_proc, patch_os, patch_which:
             with TstSuiteLoggingHandler() as log_handler:
-                ret = thin.get_tops_python("python2.7")
-                assert ret == self.exp_ret
+                exp_ret = copy.deepcopy(self.exp_ret)
+                ret = thin.get_tops_python("python3.7", ext_py_ver=[3, 7])
+                if salt.utils.platform.is_windows():
+                    for key, value in ret.items():
+                        ret[key] = str(pathlib.Path(value).resolve(strict=False))
+                    for key, value in exp_ret.items():
+                        exp_ret[key] = str(pathlib.Path(value).resolve(strict=False))
+                assert ret == exp_ret
                 assert (
-                    "ERROR:Could not auto detect file location for module concurrent for python version python2.7"
+                    "ERROR:Could not auto detect file location for module concurrent for python version python3.7"
                     in log_handler.messages
                 )
 
@@ -1159,7 +1125,6 @@ class SSHThinTestCase(TestCase):
             self._popen(
                 None,
                 side_effect=[
-                    (bts("distro.py"), bts("")),
                     (bts("tornado/__init__.py"), bts("")),
                     (bts("msgpack/__init__.py"), bts("")),
                     (bts("certifi/__init__.py"), bts("")),
@@ -1169,6 +1134,7 @@ class SSHThinTestCase(TestCase):
                     (bts(""), bts("")),
                     (bts(""), bts("")),
                     (bts(""), bts("")),
+                    (bts("distro.py"), bts("")),
                 ],
             ),
         )
@@ -1179,7 +1145,14 @@ class SSHThinTestCase(TestCase):
         patch_os = patch("os.path.exists", return_value=True)
         patch_which = patch("salt.utils.path.which", return_value=True)
         with patch_proc, patch_os, patch_which:
-            ret = thin.get_tops_python("python2.7", exclude=self.exc_libs)
+            ret = thin.get_tops_python(
+                "python3.7", exclude=self.exc_libs, ext_py_ver=[3, 7]
+            )
+            if salt.utils.platform.is_windows():
+                for key, value in ret.items():
+                    ret[key] = str(pathlib.Path(value).resolve(strict=False))
+                for key, value in exp_ret.items():
+                    exp_ret[key] = str(pathlib.Path(value).resolve(strict=False))
             assert ret == exp_ret
 
     def test_pack_alternatives_exclude(self):
@@ -1232,7 +1205,7 @@ class SSHThinTestCase(TestCase):
             thin._pack_alternative(ext_conf, self.digest, self.tar)
             calls = self.tar.mock_calls
             for _file in exp_files:
-                assert [x for x in calls if "{}".format(_file) in x.args]
+                assert [x for x in calls if "{}".format(_file) in x[-2]]
 
     def test_pack_alternatives(self):
         """
@@ -1242,11 +1215,11 @@ class SSHThinTestCase(TestCase):
             thin._pack_alternative(self.ext_conf, self.digest, self.tar)
             calls = self.tar.mock_calls
             for _file in self.exp_files:
-                assert [x for x in calls if "{}".format(_file) in x.args]
+                assert [x for x in calls if "{}".format(_file) in x[-2]]
                 assert [
                     x
                     for x in calls
-                    if os.path.join("test", "pyall", _file) in x.kwargs["arcname"]
+                    if os.path.join("test", "pyall", _file) in x[-1]["arcname"]
                 ]
 
     def test_pack_alternatives_not_normalized(self):
@@ -1260,11 +1233,11 @@ class SSHThinTestCase(TestCase):
             thin._pack_alternative(self.ext_conf, self.digest, self.tar)
             calls = self.tar.mock_calls
             for _file in self.exp_files:
-                assert [x for x in calls if "{}".format(_file) in x.args]
+                assert [x for x in calls if "{}".format(_file) in x[-2]]
                 assert [
                     x
                     for x in calls
-                    if os.path.join("test", "pyall", _file) in x.kwargs["arcname"]
+                    if os.path.join("test", "pyall", _file) in x[-1]["arcname"]
                 ]
 
     def test_pack_alternatives_path_doesnot_exist(self):
@@ -1287,11 +1260,11 @@ class SSHThinTestCase(TestCase):
                 assert msg in log_handler.messages
         calls = self.tar.mock_calls
         for _file in self.exp_files:
-            arg = [x for x in calls if "{}".format(_file) in x.args]
+            arg = [x for x in calls if "{}".format(_file) in x[-2]]
             kwargs = [
                 x
                 for x in calls
-                if os.path.join("test", "pyall", _file) in x.kwargs["arcname"]
+                if os.path.join("test", "pyall", _file) in x[-1]["arcname"]
             ]
             if "jinja2" in _file:
                 assert not arg
@@ -1329,7 +1302,7 @@ class SSHThinTestCase(TestCase):
             thin._pack_alternative(ext_conf, self.digest, self.tar)
             calls = self.tar.mock_calls
             for _file in exp_files:
-                assert [x for x in calls if "{}".format(_file) in x.args]
+                assert [x for x in calls if "{}".format(_file) in x[-2]]
 
     def test_pack_alternatives_empty_dependencies(self):
         """
@@ -1361,8 +1334,9 @@ class SSHThinTestCase(TestCase):
             thin._pack_alternative(ext_conf, self.digest, self.tar)
             calls = self.tar.mock_calls
             for _file in exp_files:
-                assert [x for x in calls if "{}".format(_file) in x.args]
+                assert [x for x in calls if "{}".format(_file) in x[-2]]
 
+    @pytest.mark.slow_test
     @skipIf(
         salt.utils.platform.is_windows(), "salt-ssh does not deploy to/from windows"
     )
@@ -1376,16 +1350,13 @@ class SSHThinTestCase(TestCase):
         # This was previously an integration test and is now here, as a unit test.
         # Should actually be a functional test
         with VirtualEnv() as venv:
-            salt.utils.thin.gen_thin(venv.venv_dir)
-            thin_dir = os.path.join(venv.venv_dir, "thin")
-            thin_archive = os.path.join(thin_dir, "thin.tgz")
-            tar = tarfile.open(thin_archive)
-            tar.extractall(thin_dir)
+            salt.utils.thin.gen_thin(str(venv.venv_dir))
+            thin_dir = venv.venv_dir / "thin"
+            thin_archive = thin_dir / "thin.tgz"
+            tar = tarfile.open(str(thin_archive))
+            tar.extractall(str(thin_dir))
             tar.close()
             ret = venv.run(
-                venv.venv_python,
-                os.path.join(thin_dir, "salt-call"),
-                "--version",
-                check=False,
+                venv.venv_python, str(thin_dir / "salt-call"), "--version", check=False,
             )
             assert ret.exitcode == 0, ret
